package agent

import (
	"net/http"

	"github.com/zeromicro/go-zero/rest/httpx"
	"tydata-server/app/user/cmd/api/internal/logic/agent"
	"tydata-server/app/user/cmd/api/internal/svc"
	"tydata-server/app/user/cmd/api/internal/types"
	"tydata-server/common/result"
	"tydata-server/pkg/lzkit/validator"
)

func GetAgentRewardsHandler(svcCtx *svc.ServiceContext) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		var req types.GetRewardsReq
		if err := httpx.Parse(r, &req); err != nil {
			result.ParamErrorResult(r, w, err)
			return
		}
		if err := validator.Validate(req); err != nil {
			result.ParamValidateErrorResult(r, w, err)
			return
		}
		l := agent.NewGetAgentRewardsLogic(r.Context(), svcCtx)
		resp, err := l.GetAgentRewards(&req)
		result.HttpResult(r, w, resp, err)
	}
}
